C######## ######### ######### ######### ######### ######### ######### ##
C  
C
C  File:DSRphase.fs
C  Author: Mihai Popovici June 93
C  Stationary phase for DSR 
C
c        from par:     integer nt=64,ny=64
c	 from par:     integer nh=64,nz=64
c	 from par:     integer np=10
c        from par:     real dt=0.01,dy=10
c        from par:     real dh=5,   dz=10
c	 from par:     real h=100
C
C######## ######### ######### ######### ######### ######### ######### ##
c%
        integer nt,nh,ny,nz

        from par:     integer nt=64,ny=64
	from par:     integer nh=64,nz=64
	from par:     integer ys=8,ts=8

	allocate: real    pdata(nh,nz),tdata(nh,nt,ns)
	allocate: complex phase(nt,ny),ph(nh,nt,ny)
	allocate: real    v(nz)

        subroutine DSRphase(nt,nh,ny,nz,v,pdata,tdata,phase,ph,h)
        integer nt,ny,nh,nz
	real    pdata(nh,nz),tdata(nh,nt,ny),v(nz)
	complex phase(nt,ny),ph(nh,nt,ny)

	integer iz,iw,iky,ikh,nw,nky,nkh,esize
        real    dt,dy,dh,dz,h,t
	real    dw,dky,dkh,w0,ky0,kh0
	real    vv,w,wa,ww2,ky,kya,kh,kha
	real    PI,J,J1,J2,k1,k2,phmax,phpp,khmax
	complex phase0
	real    signum,sigw,amp0,amp1,eps

        from par:     real dt=0.01,dy=10.
        from par:     real dh=5.,   dz=10.
	from par:     real h=100.

	eps=1.e-3
	esize=4
        call putch('n1','i',nh)
        call putch('n2','i',nz)
        call putch('n3','i',nt*ny)
        call putch('d1','f',dt)
        call putch('d2','f',dy)
        call putch('d3','f',1.)
	call putch('esize','i',4)
        call hclose()

	call auxputch('n1','i',nz,'p0')
	call auxputch('n2','i',nt*ny,'p0')
	call auxputch('n3','i',1,'p0')
	call auxputch('esize','i',4,'p0')

	call auxputch('n1','i',nz,'p1')
	call auxputch('n2','i',nt*ny,'p1')
	call auxputch('n3','i',1,'p1')
	call auxputch('esize','i',4,'p1')

        PI  =4.*atan(1.)
        nw  =nt
	nky =ny
	nkh =nh
        dw  =2.*PI/(nt*dt)
        dky =2.*PI/(ny*dy)
        dkh =2.*PI/(nh*dh)
	w0  = -PI/dt
	ky0 = -PI/dy
	kh0 = -PI/dh

	do iz=1,nz
	  v(iz)=1000.
	enddo
c	call sreed('vel',v,nz*esize)

C######## ######### ######### ######### ######### ######### ######### ##
        do iky=1,nky
	  do iw=1,nw
	    do ikh=1,nkh
              kh=kh0+(ikh-1)*dkh
	      tdata(ikh,iw,iky)=-kh*h
              ph(ikh,iw,iky)=cexp(cmplx(0.,-kh*h))
	    enddo
	  enddo
	enddo

        do iky=1,nky
	  ky=(ky0+(iky-1)*dky)*0.5
	  do iw=1,nw
	    w=w0+(iw-1)*dw
	    sigw=signum(w)
C######## ######### ######### ######### ######### ######### ######### ##
C stationary phase 
	    do iz=1,nz
	      vv=v(iz)
	      ww2=w/vv
              ww2=ww2*ww2
	      phmax=abs( tdata(1,iw,iky))

	      do ikh=1,nkh
	        kh=(kh0+(ikh-1)*dkh)*0.5
	        J1=ww2-(ky+kh)*(ky+kh)
                J2=ww2-(ky-kh)*(ky-kh)
                if(J1.gt.eps.and.J2.gt.eps) then
                  J=-sigw*(sqrt(J1)+sqrt(J2))*dz
	          tdata(ikh,iw,iky)=tdata(ikh,iw,iky)+J
	          pdata(ikh,iz)=tdata(ikh,iw,iky)
	          ph(ikh,iw,iky)=ph(ikh,iw,iky)*cexp(cmplx(0.,J))
	        else
	          pdata(ikh,iz)=0.
	          ph(ikh,iw,iky)=0.
	        endif
	        phase(iw,iky)=phase(iw,iky)+ph(ikh,iw,iky)
	        t=abs(tdata(ikh,iw,iky))
	        if(t.gt.phmax) then
	          phmax=t
	          khmax=kh
	        endif
	      enddo
C Compute theoretical stationary phase value
c 	       kha=abs(khmax)
c              if(wa.gt.kha+kya.and.mod(iw,10).eq.0) then
c	        k1=(ky-khmax)*(ky-khmax)
c	        k2=(ky+khmax)*(ky+khmax)
c	        J1=sqrt(ww2-k1)
c	        J2=sqrt(ww2-k2)
c	        phpp=1./J1+1./J2+k1/(J1*J1*J1)+k2/(J2*J2*J2)
c	        J=sqrt(8*PI/phpp)
c	        phase0=J*cexp(cmplx(0.,sigw*(-phmax+PI/4.)))
c	        amp0=abs(phase0)
c	        amp1=abs(phase(iw,iky))
c	      else
c	        amp0=0.
c	        amp1=0.
c	      endif
c	      call srite('p0',amp0,esize)
c	      call srite('p1',amp1,esize)
	    enddo
            call srite('out',pdata,nh*nz*esize)
	  enddo
	enddo

C######## ######### ######### ######### ######### ######### ######### ##
	return
	end



