C######################################################################
C  <vel.H COdatavz.x TT=traveltime.H  >COdatavz.H
C  File:COdatavz.fs
C  Author: Mihai Popovici  Feb 93
C
C  Makes constant-offset hyperbolas in v(z)
C
C       from history:   integer n1:nz, n2:nx
C       from history:   real d1:dz=25.,d2:dx=12.5
C
C       from par: integer nt=512,ny=128,nh=1
C	from par: integer nshot=10,nr=nz,ntheta=nx
C	from par: real    dt=0.01, dy=10, dh=10
C	from par: real    zdiff=100.,dzdiff=100.
C       from par: integer nzdiff=3
C
C source in /sep/mihai/dev/Vel/COdatavz.fs
C
C######## ######### ######### ######### ######### ######### ######### ##
c%

        integer nt,ny,nh,nz,nx,nzdiff
	real    dt,dy,dh,dz,dx
	real    zdiff,dzdiff

        from history:   integer n1:nz, n2:nx
        from history:   real    d1:dz, d2:dx

        from par: integer nt=512,ny=128,nh=1
	from par: integer nshot=10,nr=nz,ntheta=nx
	from par: real    dt=0.01, dy=10, dh=10
	from par: real    zdiff=100.,dzdiff=100.
	from par: integer nzdiff=1

	ntp=nt+32-1
	allocate: real COdata(nt,ny,nh)
	allocate: real data2(ntp)
	allocate: real v(nz,nx)
        allocate: real tt(nz,nx)
	allocate: real diff(nzdiff)

        subroutine COmake(dt,nt,dy,ny,dh,nh,dz,nz,dx,nx,v,diff,
        COdata,data2,zdiff,nzdiff,dzdiff,ntp,tt,nr,ntheta,nshot)

        integer nt,ny,nh,nz,nx,ntp,nr,ntheta,nshot,nzdiff
	real    COdata(nt,ny,nh)
	real    data2(ntp),diff(nzdiff)
	real    v(nz,nx),tt(nz,nx)
        real    dt,dy,dh,dz,dx
	real    ydiff,zdiff,dzdiff


	integer it,iy,ih,iz,ix,esize,np,i,iw,ix1,ix2,idiff
        real    wavelet(32)
	real    y,h,ysrc,yrec,tsrc,trec,rts,rtr
	real    x0,z0,xshot,zshot,hmax,maxoff

	data wavelet /      
     &  0.0115, 0.0543, 0.148, 0.308, 0.529, 0.776, 0.991,  1.11, 
     &  1.08,   0.897,  0.577, 0.183,-0.212,-0.541,-0.761,-0.859,
     & -0.849, -0.76,  -0.628,-0.483,-0.349,-0.238,-0.153,-0.0932,
     & -0.0536,-0.0289,-1.43e-2,-6.16e-3,-1.83e-3,3.53e-4,1.39e-3,
     &  0.00186 /

	do i=1,32
	  wavelet(i)=wavelet(i)/wavelet(8)
	enddo
        np=32
  
	esize=4
        call putch('n1','i',nt)
        call putch('n2','i',ny)
        call putch('n3','i',nh)
        call putch('o1','f',0.)
        call putch('o2','f',0.)
        call putch('o3','f',0.)
        call putch('d1','f',dt)
        call putch('d2','f',dy)
        call putch('d3','f',dh)
	call putch('esize','i',esize)
        call hclose()

	call sreed('in',v,nz*nx*esize)
	x0=0.
	z0=0.
        xshot=x0+(nshot-1)*dx
        zshot=z0
        hmax=(nh-1)*dh
        ydiff=(ny-1)*dy*0.5
        maxoff=ydiff+hmax
        if( (nx-nshot)*dx.lt.maxoff) then
          write(0,*) 'Model is not large enough (X axis)'
          return
        endif
	write(0,*) '*OK*  Max Offset < X axis',maxoff,(nx-nshot)*dx
	do iz=1,nz
	  do ix=1,nx
	    v(iz,ix)=1./v(iz,ix)
	  enddo
	enddo
        call FDTT(xshot,zshot,nx,x0,dx,nz,z0,dz,ntheta,nr,v,tt)

	call auxputch('n1','i',nz,'TT')
	call auxputch('n2','i',nx,'TT')
	call auxputch('n3','i',1,'TT')
	call auxputch('d1','f',dz,'TT')
	call auxputch('d2','f',dx,'TT')
	call auxputch('d3','f',1.,'TT')
	call srite('TT',tt,nz*nx*esize)
	
C######## ######### ######### ######### ######### ######### ######### ##
C Find indexes for depth of diffractors
      do iz=1,nzdiff
	diff(iz)=zdiff+(iz-1)*dzdiff
      enddo

      do ih=1,nh
	do iy=1,ny
	  do it=1,nt
	    COdata(it,iy,ih)=0.
	  enddo
	enddo
      enddo

C Not interpolating in y creates zig-zag patterns along offset axis
C if the grid multiples do not match (dy=dh).
      do ih=1,nh
        h=(ih-1)*dh
	do iy=1,ny
	  y=(iy-1)*dy
	  ysrc=ydiff-y-h
	  yrec=ysrc+2.*h
          ix1=nint(abs(ysrc)/dx)+nshot
          ix2=nint(abs(yrec)/dx)+nshot
	  do idiff=1,nzdiff
            iz=nint(diff(idiff)/dz)+1
            tsrc=tt(iz,ix1)
            trec=tt(iz,ix2)    
c	    rts=1./tsrc
c	    rtr=1./trec
	    rts=1./sqrt(tsrc)
	    rtr=1./sqrt(trec)
            it=nint((tsrc+trec)/dt)+1
	    if(it.ge.1.and.it.le.nt) then
              COdata(it,iy,ih)=COdata(it,iy,ih)+rts*rtr
	    endif
	  enddo
	enddo
      enddo

C Convolve the data with the wavelet
        do ih=1,nh
          do iy=1,ny
            do it=1,ntp
              data2(it)=0.
            enddo
            do it=1,nt
              do iw=1,np
                data2(it+iw-1)=data2(it+iw-1)+COdata(it,iy,ih)*
     &          wavelet(iw)
              enddo
            enddo
            do it=1,nt
              COdata(it,iy,ih)=data2(it+7)
            enddo
          enddo
        enddo

	call srite('out',COdata,nt*ny*nh*esize)

	return
	end

