C######################################################################
C  File:MZOorDMO.fs
C  Author: Mihai Popovici Jan 93
C
C       from history: integer n1:nt,n2:ny,n3:nh
C       from history: real d1:dt,d2:dy,d3:dh
C	from par: real v=2000.
C	from par: real amptype=0 My MZO
C                             =1 Hale  (input data should be NMO-ed)
C                             =2 Zhang (input data should be NMO-ed)
C                             =3 MZO from stationary phase
C                             =4 MZO from stationary phase w/o wk
C
C
C######## ######### ######### ######### ######### ######### ######### ##
c%

        integer nt,nh,ny
	real    dt,dy,dh
	real    v

        from history: integer n1:nt,n2:ny,n3:nh
        from history: real d1:dt,d2:dy,d3:dh
	from par: real v=2000.
	from par: real amptype=0
	from par: real h=0
	
	allocate: real COdata(nt,ny,nh)
	allocate: real ZOdata(nt,ny)
	allocate: complex data(nt,ny),pdata(nt,ny)

        subroutine MZObyFT(dt,nt,dh,nh,dy,ny,v,COdata,ZOdata,
        data,pdata,amptype,h)

        integer nt,ny,nh
        real    dt,dy,dh,h
	real    COdata(nt,ny,nh), ZOdata(nt,ny)
	complex data(nt,ny),pdata(nt,ny)
	real    v,amptype

	integer it,iy,ih,iw,ik,nw,ntmin,nh0,esize
	real    dw,dky,w0,ky0,tscale,thmin
	real    h,h2,hv,hvw2,w,wa,w2,k,k2,kh2,thw2
	complex phase
	real    th,dp,Jm,wk,dp1
	real    PI,PI4,PI2S,signum,eps

	esize=4
	eps=1.e-11

        call putch('n1','i',nt)
        call putch('n2','i',ny)
        call putch('d1','f',dt)
        call putch('d2','f',dy)
        call putch('d3','f',dh)
	call putch('esize','i',esize)
        call putch('n3','i',nh)
        call hclose()

	call sreed('in',COdata,nt*ny*nh*esize)

        PI  =4.*atan(1.)
	PI4 =0.25*PI
	PI2S=sqrt(2.*PI)
        nw  =nt
        dw  =2.*PI/(nt*dt)
        dky =2.*PI/(ny*dy)
	w0  = -PI/dt
	ky0 = -PI/dy
        tscale=1./sqrt(float(nt))

C######## ######### ######### ######### ######### ######### ######### ##
      if(nh.gt.1) then
        do iy=1,ny
	  do it=1,nt
	    ZOdata(it,iy)=COdata(it,iy,1)
	  enddo
        enddo
        call srite('out',ZOdata,nt*ny*esize)
	nh0=2
      else
	nh0=1
      endif
      do iy=1,ny
	do it=1,nt
	  ZOdata(it,iy)=0.
	enddo
      enddo

      do ih=nh0,nh
	if(nh0.eq.2) h=dh*(ih-1)
        h2=h*h
        hv=4.*h2/(v*v)
        thmin=2.*h/v
	ntmin=int(thmin/dt+0.5)+1
	if(ntmin.gt.nt) then
	  write(0,*) 'Time axis too short for this offset'
	  ntmin=nt
	endif
	do iy=1,ny
	  do it=1,nt
	    data(it,iy)=cmplx(COdata(it,iy,ih),0.)
	    pdata(it,iy)=0.
	  enddo
	enddo
C FFT in y
        call ft2axis( 0,-1.,nt,ny,data)

	do ik=2,ny
	  k=ky0+(ik-1)*dky
	  k2=k*k
	  kh2=k2*h2
	  do iw=2,nw
	    w=w0+(iw-1)*dw
	    wa=abs(w)
	    w2=w*w
	    hvw2=hv*w2
	    wk=4.*w2/(v*v)-k2
            if(amptype.eq.0) then
C My MZO 
	      do it=ntmin+1,nt
	        th=(it-1)*dt
	        thw2=th*th*w2
	        dp=sqrt(thw2-hvw2+kh2)
	        phase=cexp(cmplx(0.,signum(w)*dp))
	        if(dp.gt.eps) then
	          Jm=th*wa*(thw2-hvw2+2.*kh2)/(dp*dp*dp)
	        else
	          Jm=0.
	        endif
	        pdata(iw,ik)=pdata(iw,ik)+Jm*phase*data(it,ik)
	      enddo
            elseif(amptype.eq.1) then
C Hale's DMO
	      do it=1,nt
	        th=(it-1)*dt
	        thw2=th*th*w2
	        dp=sqrt(thw2+kh2)
	        phase=cexp(cmplx(0.,signum(w)*dp))
	        if(dp.gt.eps) then
	          Jm=th*wa/dp
	        else
	          Jm=0.
	        endif
	        pdata(iw,ik)=pdata(iw,ik)+Jm*phase*data(it,ik)
	      enddo
            elseif(amptype.eq.2) then
C Zhang's DMO
	      do it=1,nt
	        th=(it-1)*dt
	        thw2=th*th*w2
	        dp=sqrt(thw2+kh2)
	        phase=cexp(cmplx(0.,signum(w)*dp))
	        if(dp.gt.eps) then
	          Jm=th*wa*(thw2+2.*kh2)/(dp*dp*dp)
	        else
	          Jm=0.
	        endif
	        pdata(iw,ik)=pdata(iw,ik)+Jm*phase*data(it,ik)
	      enddo
            elseif(amptype.eq.3) then
C MZO from stationary phase
	      if(wk.ge.0.) then
	      wk=sqrt(wk)
	      do it=ntmin+1,nt
	        th=(it-1)*dt
	        thw2=th*th*w2
	        dp=sqrt(thw2-hvw2+kh2)
	        phase=cexp(cmplx(0.,signum(w)*(dp+PI4)))
	        if(dp.gt.eps) then
	          dp1=sqrt(dp)
	          Jm=(thw2-hvw2)*wk*PI2S/(dp*dp*dp1)
	        else
	          Jm=0.
	        endif
	        pdata(iw,ik)=pdata(iw,ik)+Jm*phase*data(it,ik)
	      enddo
	      endif
            elseif(amptype.eq.4) then
C MZO from stationary phase with other amplitude
	      do it=ntmin+1,nt
	        th=(it-1)*dt
	        thw2=th*th*w2
	        dp=sqrt(thw2-hvw2+kh2)
	        phase=cexp(cmplx(0.,signum(w)*(dp+PI4)))
	        if(dp.gt.eps) then
	          dp1=sqrt(dp)
	          Jm=(thw2-hvw2)*PI2S/(dp*dp*dp1)
	        else
	          Jm=0.
	        endif
	        pdata(iw,ik)=pdata(iw,ik)+Jm*phase*data(it,ik)
	      enddo
            endif
	  enddo
	enddo
C IFFT in y
        call ft2axis( 1,-1.,nt,ny,pdata)
C IFFT in t 
        call ft1axis( 1, 1.,nt,ny,pdata)
	do iy=1,ny
	  do it=1,nt
	    ZOdata(it,iy)=real(pdata(it,iy))*tscale
	  enddo
	enddo
	call srite('out',ZOdata,nt*ny*esize)
      enddo
C end loop in nh

      return
      end

