C######## ######### ######### ######### ######### ######### ######### ##
C
C  File:MZOphasetab.fs
C  Author: Mihai Popovici Oct 94
C  Linear phase for MZO
C
c        from par:     integer nt=64,ny=64
c	 from par:     integer nh=64
c	 from par:     integer np=10
c        from par:     real dt=0.01,dy=10
c        from par:     real dh=5
c	 from par:     real h=100
C
C######## ######### ######### ######### ######### ######### ######### ##
c%
        integer nt,nh,ny,ts,ys

        from par:     integer nt=64,ny=64
	from par:     integer nh=64
	from par:     integer ys=8,ts=8

	allocate: real    v(nt)
	allocate: real    pdata(nh,nt),tdata(nh,nt,ny)
	allocate: complex ph(nh,nt)

        subroutine DSRphase(nt,nh,ny,v,pdata,tdata,ph,h,ys,ts)
        integer nt,ny,nh,ys,ts
	real    pdata(nh,nt),tdata(nh,nt,ny),v(nt)
	complex ph(nh,nt)

	integer it,iw,iky,ikh,nw,nky,nkh,esize
        real    dt,dy,dh,dtv,h,khta,t
	real    dw,dky,dkh,w0,ky0,kh0,ww2,ky2
	real    vv,w,wa,wy,ky,kya,kh,kha
	real    PI,J
	real    signum,sigw

        from par:     real dt=0.01,dy=10
        from par:     real dh=5
	from par:     real h=100

	esize=4
        call putch('n1','i',nh)
        call putch('n2','i',nt)
        call putch('n3','i',nt/ts*ny/ys)
        call putch('d1','f',dt)
        call putch('d2','f',dy)
        call putch('d3','f',1)
	call putch('esize','i',4)
        call hclose()

	call auxputch('n1','i',nh,'fft')
	call auxputch('n2','i',nt,'fft')
        call auxputch('n3','i',nt/ts*ny/ys,'fft')
        call auxputch('d1','f',dt,'fft')
        call auxputch('d2','f',dy,'fft')
        call auxputch('d3','f',1,'fft')
	call auxputch('esize','i',4,'fft')

        PI  =4.*atan(1.)
        nw  =nt
	nky =ny
	nkh =nh
        dw  =2.*PI/(nt*dt)
        dky =2.*PI/(ny*dy)
        dkh =2.*PI/(nh*dh)
	w0  = -PI/dt
	ky0 = -PI/dy
	kh0 = -PI/dh

	do it=1,nt
	  v(it)=2000.
	enddo
c	call sreed('vel',v,nt*esize)

C######## ######### ######### ######### ######### ######### ######### ##
        do iky=1,nky
	  do iw=1,nw
	    do ikh=1,nkh
              kh=kh0+(ikh-1)*dkh
	      tdata(ikh,iw,iky)=-kh*h
c	      tdata(ikh,iw,iky)=0.
	    enddo
	  enddo
	enddo

        do iky=1,nky
	  ky=(ky0+(iky-1)*dky)*0.5
	  kya=abs(ky)
	  do iw=1,nw
	    w=w0+(iw-1)*dw
	    sigw=signum(w)
	    do it=1,nt
              t=it*dt
	      vv=v(it)
	      dtv=vv*dt*0.5
	      wa=abs(w)/vv
              wy=abs(wa-kya)
	      do ikh=1,nkh
	        kh=(kh0+(ikh-1)*dkh)*0.5
	        kha=abs(kh)
	        if(wy.ge.kha) then
	          J=-sigw*(sqrt((wa-kya)*(wa-kya)-kh*kh)+
     &            sqrt((wa+kya)*(wa+kya)-kh*kh))*dtv
	          tdata(ikh,iw,iky)=tdata(ikh,iw,iky)+J
	          pdata(ikh,it)=tdata(ikh,iw,iky)
	        else
                  tdata(ikh,iw,iky)=0.
	          pdata(ikh,it)=0.
	        endif
	      enddo
            enddo
	    if(mod(iw,ts).eq.0.and.mod(iky,ys).eq.0) then
              call srite('out',pdata,nh*nt*esize)
	    endif
	  enddo
	enddo

C######## ######### ######### ######### ######### ######### ######### ##
C stationary phase 
        do iky=1,nky
	  do iw=1,nw
	    do ikh=1,nkh
              kh=kh0+(ikh-1)*dkh
	      tdata(ikh,iw,iky)=-kh*h
	    enddo
	  enddo
	enddo

        do iky=1,nky
	  ky=ky0+(iky-1)*dky
          ky2=ky*ky
	  kya=abs(ky)
	  do iw=1,nw
	    w=w0+(iw-1)*dw
	    sigw=signum(w)
            ww2=4.*w*w/(vv*vv)
	    wa=abs(w)
	    do it=1,nt
              t=it*dt
	      vv=v(it)
	      do ikh=1,nkh
	        kh=kh0+(ikh-1)*dkh
	        if(ww2.gt.ky2) then
	          J=-sigw*wa*t*0.5*(kh*kh/(ww2-ky2)+2.)
c	          tdata(ikh,iw,iky)=tdata(ikh,iw,iky)+J
	          tdata(ikh,iw,iky)=J-h*kh
	          pdata(ikh,it)=tdata(ikh,iw,iky)
	        else
                  tdata(ikh,iw,iky)=0.
	          pdata(ikh,it)=0.
	        endif
	      enddo
            enddo
	    if(mod(iw,ts).eq.0.and.mod(iky,ys).eq.0) then
              call srite('fft',pdata,nh*nt*esize)
	    endif
	  enddo
	enddo



c              do it=1,nt
c                t=it*dt
c                if(wa.ne.0.) then
c                  khta=h*4.*(wa*wa-ky*ky)/(wa*t)
c                else
c                  khta=0.
c                endif
c	        if(wy.ge.abs(khta)) then
c                  J=-sigw*(sqrt((wa-kya)*(wa-kya)-khta*khta)+
c     &            sqrt((wa+kya)*(wa+kya)-khta*khta))*t*vv*0.5
c                else
c                  J=0.
c                endif
c	        write(0,*) khta,J
c                pdata(nh+1,it)=J
c              enddo

C######## ######### ######### ######### ######### ######### ######### ##
	return
	end



